/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2019-2021, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_morm_samples_person
#define headerfile_morm_samples_person

#include <string>
#include "morm.h"
#include "language.h"
#include "attachment.h"
#include "attachment2.h"



namespace morm
{
namespace samples
{

/*
CREATE TABLE public.person (
    id bigserial,
    first_name varchar(64),
    last_name varchar(64),
    email varchar(64),
    language_id bigint,

    primary key(id)
);
*/

class Person : public morm::Model
{
public:

	long id;

	std::wstring first_name;
	std::wstring last_name;
	std::wstring email;
	Language language;

	std::list<Attachment> attachments;
	Attachment2 attachment2;


	void map_fields()
	{
		field(L"id", id, FT::no_insertable | FT::no_updatable | FT::primary_key);
		field(L"language_id", L"language", language, FT::foreign_key);
		field(L"first_name", first_name);
		field(L"last_name", last_name);
		field(L"email", email);
		field(L"person_id", L"attachments", attachments, FT::foreign_key_in_child);
		field(L"person_id", L"attachment2", attachment2, FT::foreign_key_in_child | FT::no_insertable | FT::no_updatable);
	}


	void prepare_table()
	{
		table(L"public", L"person");
	}


	void after_select()
	{
		if( has_primary_key_set )
		{
			morm::Finder<Attachment> finder(model_connector);
			attachments = finder.select().where().eq(L"person_id", id).get_list();

			morm::Finder<Attachment2> finder2(model_connector);
			attachment2 = finder2.select().where().eq(L"person_id", id).get();
		}
	}

	void after_insert()
	{
		get_last_sequence_for_primary_key(L"public.person_id_seq", id);
	}


};



}
}


#endif

